@@ -33,6 +33,7 @@
 #include <QDir>
 
 #include <algorithm>
+#include <filesystem>
 
 using namespace  KSysGuard;
 
@@ -368,7 +369,7 @@ void CGroupDataModel::setRoot(const QString &root)
     d->m_root = root;
     d->m_rootGroup.reset(new CGroup(root));
     emit rootChanged();
-    update();
+    QMetaObject::invokeMethod(this, [this] {update();}, Qt::QueuedConnection);
 }
 
 void CGroupDataModel::update()
@@ -409,22 +410,22 @@ bool CGroupDataModel::filterAcceptsCGroup(const QString &id)
 
 void CGroupDataModel::update(CGroup *node)
 {
+    namespace fs = std::filesystem;
     const QString path = CGroup::cgroupSysBasePath() + node->id();
-    QDir dir(path);
-    if (!dir.exists()) {
-        return;
-    }
 
     // Update our own stat info
     // This may trigger some dataChanged
     node->requestPids(this, [this, node]() {
         auto row = d->m_cGroups.indexOf(node);
-        Q_EMIT dataChanged(index(row, 0, QModelIndex()), index(row, 0, QModelIndex()));
+        Q_EMIT dataChanged(index(row, 0, QModelIndex()), index(row, columnCount()-1, QModelIndex()));
     });
 
-    const auto entries = dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
-    for (auto subDir : entries) {
-        const QString childId = node->id() % QLatin1Char('/') % subDir;
+
+    for (const auto& entry : fs::directory_iterator(path.toUtf8().data())) {
+        if (!entry.is_directory()) {
+            continue;
+        }
+        const QString childId = node->id() % QLatin1Char('/') % QString::fromUtf8(entry.path().filename().c_str());
         CGroup *childNode = d->m_cgroupMap[childId];
         if (!childNode) {
             childNode = new CGroup(childId);
